package com.hys.app.service.erp.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.poi.excel.ExcelUtil;
import cn.hutool.poi.excel.ExcelWriter;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hys.app.converter.erp.WarehouseOutConverter;
import com.hys.app.converter.erp.WarehouseOutItemConverter;
import com.hys.app.framework.database.BaseQueryParam;
import com.hys.app.framework.database.WebPage;
import com.hys.app.framework.exception.ServiceException;
import com.hys.app.framework.database.mybatisplus.base.BaseServiceImpl;
import com.hys.app.framework.rabbitmq.MessageSender;
import com.hys.app.framework.rabbitmq.MqMessage;
import com.hys.app.framework.util.BeanUtils;
import com.hys.app.framework.util.DateUtil;
import com.hys.app.framework.util.PageConvert;
import com.hys.app.framework.util.StringUtil;
import com.hys.app.mapper.erp.WarehouseOutMapper;
import com.hys.app.mapper.goods.GoodsMapper;
import com.hys.app.model.base.rabbitmq.AmqpExchange;
import com.hys.app.model.erp.dos.*;
import com.hys.app.model.erp.dto.*;
import com.hys.app.model.erp.dto.message.WarehouseOutShipMessage;
import com.hys.app.model.erp.enums.*;
import com.hys.app.model.erp.vo.*;
import com.hys.app.model.system.dos.AdminUser;
import com.hys.app.model.system.dos.LogisticsCompanyDO;
import com.hys.app.service.erp.*;
import com.hys.app.service.kuaidiniao.KuaiDiNiaoService;
import com.hys.app.service.kuaidiniao.entity.PreOrderModel;
import com.hys.app.service.system.AdminUserManager;
import com.hys.app.service.system.LogisticsCompanyManager;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.hys.app.framework.util.CollectionUtils.*;

/**
 * 出库单业务层实现
 *
 * @author 张崧
 * @since 2023-12-07 16:50:20
 */
@Service
public class WarehouseOutManagerImpl extends BaseServiceImpl<WarehouseOutMapper, WarehouseOutDO> implements WarehouseOutManager {

    @Autowired
    private WarehouseOutConverter converter;

    @Autowired
    private OrderManager orderManager;

    @Autowired
    private NoGenerateManager noGenerateManager;

    @Autowired
    private WarehouseOutItemManager warehouseOutItemManager;

    @Autowired
    private WarehouseEntryBatchManager batchManager;

    @Autowired
    private WarehouseOutMapper warehouseOutMapper;

    @Autowired
    private MessageSender messageSender;

    @Autowired
    private AdminUserManager adminUserManager;

    @Autowired
    private GoodsMapper goodsMapper;

    @Autowired
    private FinanceItemManager financeItemManager;


    @Autowired
    private ProductStockManager productStockManager;


    @Override
    public WebPage<WarehouseOutVO> list(WarehouseOutQueryParams queryParams) {
        WebPage<WarehouseOutDO> webPage = baseMapper.selectPage(queryParams);
        WebPage<WarehouseOutVO> convert = converter.convert(webPage);
        return convert;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long add(WarehouseOutDTO warehouseOutDTO) {
        // 保存出库单
        WarehouseOutDO warehouseOutDO = BeanUtil.copyProperties(warehouseOutDTO, WarehouseOutDO.class);
        warehouseOutDO.setSn(noGenerateManager.generate(NoBusinessTypeEnum.WarehouseOut, warehouseOutDO.getDeptId()));
        warehouseOutDO.setCreateBy(adminUserManager.getCurrUserName());
        save(warehouseOutDO);
        // 保存出库单明细
        warehouseOutDTO.setId(warehouseOutDO.getId());
        List<WarehouseOutItemDO> productList = BeanUtils.toBean(warehouseOutDTO.getProductList(), WarehouseOutItemDO.class);
        productList.forEach(item->item.setWarehouseOutId(warehouseOutDO.getId()));
        warehouseOutItemManager.saveBatch(productList);
        //如果是出库就要扣减库存
        if ("submit".equals(warehouseOutDTO.getSaveOrSubmit())) {
            updateStock(warehouseOutDTO, warehouseOutDO);
        }
        return warehouseOutDO.getId();
    }


    @Override
    public void edit(WarehouseOutDTO warehouseOutDTO) {
        // 保存出库单
        WarehouseOutDO warehouseOutDO = BeanUtil.copyProperties(warehouseOutDTO, WarehouseOutDO.class);
        updateById(warehouseOutDO);
        // 保存出库单明细
        List<WarehouseOutItemDO> productList = BeanUtils.toBean(warehouseOutDTO.getProductList(), WarehouseOutItemDO.class);
        productList.forEach(item->item.setWarehouseOutId(warehouseOutDO.getId()));
        warehouseOutItemManager.deleteByWarehouseOutId(Collections.singletonList(warehouseOutDTO.getId()));
        warehouseOutItemManager.saveBatch(productList);
        //如果是出库就要扣减库存
        if ("submit".equals(warehouseOutDTO.getSaveOrSubmit())) {
            updateStock(warehouseOutDTO, warehouseOutDO);
        }
    }

    private void updateStock(WarehouseOutDTO warehouseOutDTO, WarehouseOutDO warehouseOutDO) {
        if ("Y".equals(warehouseOutDO.getHasSubmit())) {
            throw new ServiceException("数据已出库,请勿重复操作");
        }
        List<Long> collect = warehouseOutDTO.getProductList().stream().map(WarehouseOutProductDTO::getGoodsId).collect(Collectors.toList());
        List<ProductStockDO> stockDOList = productStockManager.list(new LambdaQueryWrapper<ProductStockDO>()
                .eq(ProductStockDO::getWarehouseId, warehouseOutDTO.getWarehouseId()).in(ProductStockDO::getGoodsId, collect));
        for (ProductStockDO aDo : stockDOList) {
            WarehouseOutProductDTO productDO = warehouseOutDTO.getProductList().stream().filter(item -> item.getGoodsId().equals(aDo.getGoodsId())).findFirst().get();
            if (productDO.getOutNum() > aDo.getUsableStock()) {
                throw new ServiceException(productDO.getGoodName() + "的可用库存不够，请核对数据");
            }
            //循环判断每一条数据可用库存够不够扣减
            aDo.setUsableStock(aDo.getUsableStock() - productDO.getOutNum());
            aDo.setActualStock(aDo.getActualStock() - productDO.getOutNum());
        }
        productStockManager.updateBatchById(stockDOList);
        //入库之后数据不能被操作
        warehouseOutDO.setHasSubmit("Y");
        updateById(warehouseOutDO);
    }

    @Override
    public WarehouseOutVO getDetail(Long id) {
        WarehouseOutDO warehouseOutDO = getById(id);
        WarehouseOutVO outVO = BeanUtil.copyProperties(warehouseOutDO, WarehouseOutVO.class);
        // 查询出库明细
        List<WarehouseOutItemDO> itemList = warehouseOutItemManager.list(new LambdaQueryWrapper<WarehouseOutItemDO>().eq(WarehouseOutItemDO::getWarehouseOutId,id));
        List<WarehouseOutProductDTO> productList = BeanUtils.toBean(itemList, WarehouseOutProductDTO.class);

        List<Long> goodIdlist = itemList.stream().map(WarehouseOutItemDO::getGoodsId).collect(Collectors.toList());

        //所有的商品信息
        List<GoodsDO> goodsDOS = goodsMapper.selectList(new LambdaQueryWrapper<GoodsDO>().in(GoodsDO::getId, goodIdlist));
        List<ProductStockDO> stockDOList = productStockManager.list(new LambdaQueryWrapper<ProductStockDO>()
                .eq(ProductStockDO::getWarehouseId, warehouseOutDO.getWarehouseId()).in(ProductStockDO::getGoodsId, goodIdlist));
        for (WarehouseOutProductDTO dto : productList) {
            GoodsVO goodsVO = BeanUtil.copyProperties(goodsDOS.stream().filter(item -> item.getId().equals(dto.getGoodsId())).findFirst().get(), GoodsVO.class);
            goodsVO.setUsableStock(stockDOList.stream().filter(item -> item.getGoodsId().equals(dto.getGoodsId())).findFirst().get().getUsableStock());
            dto.setGoodsVO(goodsVO);
        }
        outVO.setProductList(productList);
        return outVO;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long id) {
        WarehouseOutDO aDo = getById(id);
        if ("Y".equals(aDo.getHasSubmit())) {
            throw new ServiceException("该数据已提交，不能删除");
        }
        removeById(id);
        // 删除明细
        warehouseOutItemManager.remove(new LambdaQueryWrapper<WarehouseOutItemDO>().eq(WarehouseOutItemDO::getWarehouseOutId, id));
    }

    @Override
    public WarehouseOutVO preview(WarehouseOutPreviewDTO warehouseOutPreviewDTO) {
//        // 校验订单编号是否正确
//        List<OrderDO> orderList = checkOrderData(warehouseOutPreviewDTO.getOrderIdList());
//
//        // 查询订单明细
//        List<OrderItemDO> orderItemList = orderItemManager.listByOrderIds(warehouseOutPreviewDTO.getOrderIdList());
//
//        // 查询这些商品的可用入库批次
//        List<Long> productIds = convertList(orderItemList, OrderItemDO::getProductId);
//        Map<Long, List<WarehouseEntryBatchDO>> warehouseBatchMap = batchManager.listAvailableBatch(orderList.get(0).getWarehouseId(), productIds);
//        return converter.convertPreview(orderList, orderItemList, warehouseBatchMap);
        return null;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void ship(WarehouseOutShipDTO shipDTO) {
        Long id = shipDTO.getId();
        WarehouseOutDO warehouseOutDO = getById(id);
        if (!new WarehouseOutAllowable(warehouseOutDO).getShip()) {
            throw new ServiceException("当前状态不能进行发货操作");
        }
        // 扣减批次库存
        List<WarehouseOutItemDO> productList = warehouseOutItemManager.listByWarehouseOutId(id);
        List<StockUpdateDTO> stockUpdateList = converter.convertStockUpdateList(productList);
        batchManager.updateStock(StockChangeSourceEnum.WAREHOUSE_OUT, warehouseOutDO.getSn(), stockUpdateList);

        // 更新出库单信息
        lambdaUpdate()
                .set(WarehouseOutDO::getStatus, WarehouseOutStatusEnum.SHIP)
                .set(WarehouseOutDO::getShipTime, DateUtil.getDateline())
                .set(WarehouseOutDO::getLogisticsCompanyId, shipDTO.getLogisticsCompanyId())
                .set(WarehouseOutDO::getLogisticsCompanyName, shipDTO.getLogisticsCompanyName())
                .set(WarehouseOutDO::getTrackingNumber, shipDTO.getTrackingNumber())
                .set(WarehouseOutDO::getFreightPrice, shipDTO.getFreightPrice())
                .set(WarehouseOutDO::getConsignee, shipDTO.getConsignee())
                .eq(WarehouseOutDO::getId, id)
                .update();

        // 将订单状态更新为已发货
        orderManager.warehouseOutShip(shipDTO);

        // 生成财务明细
        financeItemManager.addExpand(FinanceExpandTypeEnum.Logistics, warehouseOutDO.getSn(), shipDTO.getFreightPrice());

        // 发送出库单发货消息
        WarehouseOutShipMessage message = new WarehouseOutShipMessage();
        message.setId(id);
        message.setItemList(productList);
        this.messageSender.send(new MqMessage(AmqpExchange.WAREHOUSE_OUT_SHIP, AmqpExchange.WAREHOUSE_OUT_SHIP + "_ROUTING", message));
    }

    /**
     * 查询出库单统计分页列表数据
     *
     * @param params 查询参数
     * @return
     */
    @Override
    public WebPage statistics(WarehouseOutStatisticsParam params) {
        IPage<WarehouseOutStatistics> iPage = this.warehouseOutMapper.selectWarehouseOutPage(new Page(params.getPageNo(), params.getPageSize()), params);
        return PageConvert.convert(iPage);
    }

    /**
     * 导出出库单统计列表
     *
     * @param response
     * @param params   查询参数
     */
    @Override
    public void export(HttpServletResponse response, WarehouseOutStatisticsParam params) {
        //查询出库单商品列表
        List<WarehouseOutStatistics> list = this.warehouseOutMapper.selectWarehouseOutList(params);

        ArrayList<Map<String, Object>> rows = CollUtil.newArrayList();
        ExcelWriter writer = ExcelUtil.getWriter(true);
        for (WarehouseOutStatistics statistics : list) {
            Map<String, Object> map = new LinkedHashMap<>();
            map.put("所属部门", statistics.getDeptName());
            map.put("仓库名称", statistics.getWarehouseName());
            map.put("服务专员", statistics.getDistributionName());
            map.put("出库单编号", statistics.getSn());
            map.put("出库时间", DateUtil.toString(statistics.getOutTime(), "yyyy-MM-dd HH:mm:ss"));
            map.put("入库单编号", statistics.getWarehouseEntrySn());
            map.put("商品编号", statistics.getProductSn());
            map.put("商品名称", statistics.getProductName());
            map.put("商品类别", statistics.getCategoryName());
            map.put("规格型号", statistics.getProductSpecification());
            map.put("单位", statistics.getProductUnit());
            map.put("出库数量", statistics.getOutNum());
            rows.add(map);
        }

        writer.write(rows, true);

        ServletOutputStream out = null;
        try {
            out = response.getOutputStream();
        } catch (IOException e) {
            e.printStackTrace();
        }
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet;charset=utf-8");
        response.setHeader("Content-Disposition", "attachment;filename=warehouse_out.xlsx");
        writer.flush(out, true);
        writer.close();
        IoUtil.close(out);
    }

    @Override
    public long countByWarehouseId(Long warehouseId) {
        return lambdaQuery().eq(WarehouseOutDO::getWarehouseId, warehouseId).count();
    }

    @Override
    public void audit(List<Long> ids, WarehouseOutStatusEnum status, String remark) {
        if (status != WarehouseOutStatusEnum.AUDIT_PASS && status != WarehouseOutStatusEnum.AUDIT_REJECT) {
            throw new ServiceException("审核参数错误");
        }

        List<WarehouseOutDO> list = listByIds(ids);
        for (WarehouseOutDO warehouseOutDO : list) {
            if (!new WarehouseOutAllowable(warehouseOutDO).getAudit()) {
                throw new ServiceException(warehouseOutDO.getSn() + "不能进行审核操作");
            }
        }

        AdminUser currUser = adminUserManager.getCurrUser();
        lambdaUpdate()
                .set(WarehouseOutDO::getStatus, status)
                .set(WarehouseOutDO::getAuditById, currUser.getId())
                .set(WarehouseOutDO::getAuditByName, currUser.getRealName())
                .in(WarehouseOutDO::getId, ids)
                .update();
    }



    @Override
    public Page<GoodsVO> getGoodByWarehouse(Long warehouseId, BaseQueryParam queryParam) {
        IPage<ProductStockDO> page = new Page<>(queryParam.getPageNo(), queryParam.getPageSize());
        IPage<ProductStockDO> page1 = productStockManager.page(page, new LambdaQueryWrapper<ProductStockDO>()
                .eq(ProductStockDO::getWarehouseId, warehouseId)
                .gt(ProductStockDO::getUsableStock, 0));
        List<Long> collect = page1.getRecords().stream().map(ProductStockDO::getGoodsId).collect(Collectors.toList());
        if (CollectionUtil.isNotEmpty(collect)) {
            List<GoodsDO> goodsDOS = goodsMapper.selectList(new LambdaQueryWrapper<GoodsDO>().in(GoodsDO::getId, collect));
            List<GoodsVO> beanVo = BeanUtils.toBean(goodsDOS, GoodsVO.class);
            for (GoodsVO goodsVO : beanVo) {
                goodsVO.setUsableStock(page1.getRecords().stream().filter(item -> item.getGoodsId().equals(goodsVO.getId())).findFirst().get().getUsableStock());
            }
            Page<GoodsVO> pageVo = new Page<>(queryParam.getPageNo(), queryParam.getPageNo());
            pageVo.setPages(page1.getPages());
            pageVo.setSize(page1.getSize());
            pageVo.setTotal(page1.getTotal());
            pageVo.setRecords(beanVo);
            return pageVo;
        }
        return new Page<>();
    }
}

