package com.hys.app.service.kuaidiniao;

import cn.hutool.core.codec.Base64;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.URLUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.crypto.digest.DigestUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.hys.app.framework.context.user.AdminUserContext;
import com.hys.app.framework.exception.ServiceException;
import com.hys.app.framework.util.DateUtil;
import com.hys.app.service.kuaidiniao.entity.LongitudeLatitude;
import com.hys.app.service.kuaidiniao.entity.PreOrderModel;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Map;

@Service
public class KuaiDiNiaoServiceImpl implements KuaiDiNiaoService {

    /**
     * 快递鸟apiId
     */
    @Value("${kuaidianiao.id}")
    private String id;
    /**
     * 快递鸟apiKey
     */
    @Value("${kuaidianiao.key}")
    private String apiKey;
    /**
     * 快递鸟请求地址
     */
    @Value("${kuaidianiao.apiUrl}")
    private String url;


    @Override
    public String preOrder(PreOrderModel preOrderModel) {
        //填充参数
        fillParam(preOrderModel);
        return sendUrl(JSONUtil.toJsonStr(preOrderModel), KdConstants.preOrder);
    }

    /**
     * @param dataJson      业务参数
     * @param interfaceType 请求接口指令
     * @return 返回JSON
     */
    private String sendUrl(String dataJson, String interfaceType) {
        try {
            String design = URLEncoder.encode(Base64.encode(DigestUtil.md5Hex(dataJson + apiKey)), "UTF-8");
            //拼接参数
            String param = "interfaceType=" + interfaceType + "&customerCode=" + id + "&requestData=" + dataJson + "&dataSign=" + design;
            //发请求
            String res = HttpUtil.post(url, param);
            Map map = JSON.parseObject(res, Map.class);
            //resultCode == 100表示预下单成功，预下单郑工后立马调用下单接口
            if (ObjectUtil.isNotNull(map.get("resultCode")) && 100 == Integer.valueOf(map.get("resultCode").toString())) {
                //拼接参数
                String createParam = "interfaceType=" + KdConstants.createOrder + "&customerCode=" + id + "&requestData=" + dataJson + "&dataSign=" + design;
                //发请求
                String resCreate = HttpUtil.post(url, createParam);
                Map mapCreate = JSON.parseObject(resCreate, Map.class);
                //resultCode == 100表示下单成功，预下单成功后立马调用下单接口
                if (ObjectUtil.isNotNull(mapCreate.get("resultCode")) && 100 == Integer.valueOf(mapCreate.get("resultCode").toString())) {
                    Map mapData = JSON.parseObject(mapCreate.get("data").toString(), Map.class);
                    return mapData.get("kdnOrderCode").toString();
                }
            }else {
                throw new ServiceException(map.get("reason").toString());
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new ServiceException("请求快递鸟发货异常，请联系客服");
        }
        return null;
    }

    private void fillParam(PreOrderModel preOrderModel) {
        //填充送货人的经纬度
        LongitudeLatitude longitudeLatitudeSender = GaoDeUtil.getLongitudeLatitude(preOrderModel.getSender().getProvinceCityAreaAddress());
        preOrderModel.getSender().setLongitude(longitudeLatitudeSender.getLongitude());
        preOrderModel.getSender().setLatitude(longitudeLatitudeSender.getLatitude());
        //填充收货人的经纬度
        LongitudeLatitude longitudeLatitudeReceiver = GaoDeUtil.getLongitudeLatitude(preOrderModel.getSender().getProvinceCityAreaAddress());
        preOrderModel.getReceiver().setLongitude(longitudeLatitudeReceiver.getLongitude());
        preOrderModel.getReceiver().setLatitude(longitudeLatitudeReceiver.getLatitude());
        //获取下单人信息
        preOrderModel.setOrderName(AdminUserContext.getAdminUserName());
        preOrderModel.setOrderPhone("13127911751");


    }
}
